program Solv
        
        use sep
        use matmult
        use nlsolver_mod
        use prob_l_l2_diff_mod

        implicit none

        real, dimension(:), allocatable :: yin, yout, xin, xout
        integer                         :: n1, in1, n2, in2, n3, in3, stat
        integer                         :: spexMin, spexMax, ispex, Sfmx
        real                            :: o3, d3 
        real, dimension(:), pointer :: bandlimits, alldata, alldataout
        real, dimension(:), allocatable :: modl, allslopesout
       
        integer :: buffer
        !Initialize and read in data****************************************** 
        call sep_init()
        call from_history("n1",n1)        
        call from_history("n2",n2)
        call from_history("n1",n3)
        call from_history("o3",o3)
        call from_history("d3",d3)
        allocate(alldata(n1*n2*n3)) !alldata is the wrapped ratio data
        allocate(alldataout(n1*n2*n3)) !alldataout is wrapped linearly regressed ratio data
        allocate(allslopesout(n1*n2)) !allslopesout is wrapped linearlt regressed slope in each window
        allocate(bandlimits(n1*n2*2)) !bandlimits holds the bandpass max and min
                                        !for each window
        call sep_read(alldata)
        call sep_read(bandlimits,"limits")
        allocate(xout(n3)) !xout is the full length of the freq spectrum with
                                !we will run the forward regression with
        do ispex=1, n3
                xout(ispex) = o3 + (ispex-1)*d3;
        end do
        allocate(yout(n3)) !holds amplitude values of forward regression
        allocate(modl(2))
        
        !*********************************************************************
        !for each window select ratio data only within defined range. Then
        !determine model parameters of a linear regression over the ratio data
        !within that range. Then perform forward regression to produce line over
        !entire freq range. 
        do in1=1, n1
                do in2=1, n2
                        !get min and max spectrum indices
                        spexMin=bandlimits((in1-1)+(in2-1)*n1+1)
                        spexMax=bandlimits((in1-1)+(in2-1)*n1+n1*n2+1)

                        !allocate space based on min and max indices
                        allocate(yin(spexMax-spexMin+1))
                        allocate(xin(spexMax-spexMin+1))
                                
                        !grab ratio data within bandwidth
                        do ispex=spexMin, spexMax
                                yin(ispex-spexMin+1)=alldata((in1-1)+(in2-1)*n1+ispex*n1*n2+1)
                                xin(ispex-spexMin+1)=o3+ispex*d3
                        end do
                        print*,"mx: ", in1, "win: ", in2
                        !determine model parameters of linear regression
                        modl=linreg(yin,xin)
                        !forward regression to produce line
                        yout = linRegForward(modl,xout)
                               
                        !put slope result into allslopesout
                        allslopesout((in1-1)+(in2-1)*n1)=modl(1)
                        !put into alldataout that holds line data for each
                        !window
                        do in3=1, n3
                                alldataout((in1-1)+(in2-1)*n1+(in3-1)*n1*n2+1)=yout(in3)
                                !write(0,*) "in3:",in3,"yout:",yout(in3)
                        end do
                        deallocate(yin)
                        deallocate(xin)
                end do
        end do
        
        call sep_write(allslopesout,"slopeFile")
        call sep_write(alldataout)

contains 
        function linReg(y,x)
                !linear regression solver. 
                !y = mx + b <= system to solve
                ! model=[m,b] 
                ! data =[y1, y2, ..., yn]
                ! operator G = [x1 1, x2 1, ... , xn 1]

                !yin can be thought as y values and xin as x values
                real, dimension(2)              :: linReg
                real, dimension(:), allocatable :: y,x 
                integer                         :: stat, nmodl, ndata, i
                real, dimension(:), allocatable,target :: modl
                real, dimension(:,:), pointer    :: aa
                type(prob_l_l2_diff)            :: prob

                !ensure the number of x and y values are the same
                if( size(y).neqv.size(x)) then 
                        print*,"y and x must be same size"
                        stop
                end if
                nmodl = 2 !two model parameters m and b    
                ndata = size(y) 
               
                !aa used to make operator. Same form as G described above 
                allocate(aa(ndata,2))
                do i=1, ndata
                        aa(i,1)=x(i) !first column are x values
                        aa(i,2)=1       !second column just ones
                end do
                call matmult_init(aa)

                allocate(modl(nmodl))
                modl=(/1,1/)    !starting model at arbitrary values

                prob = prob_l_l2_diff(lop=matmult_lop,nmodl=nmodl,ndata=ndata,data=y)
                stat = nlsolver(modl,prob)
                linReg = modl; 
                print*,"slope: ", linReg 
        end function         

        function linRegForward(modl,x)
                !linear regression modeler 
                !y = mx + b <= system to model
                ! model=[m,b] 
                ! data =[y1, y2, ..., yn]
                ! operator G = [x1 1, x2 1, ... , xn 1]
                
                real, dimension(:), allocatable :: x, linRegForward
                real, dimension(:), allocatable :: modl
                real, dimension(:,:), pointer    :: aa
                integer                         :: i,j 
                
                print*,modl
                allocate(linRegForward(size(x)))
                allocate(aa(size(x),2))

                do i=1, size(x)
                        aa(i,1)=x(i)
                        aa(i,2)=1
                end do

                !call matmult_init(aa)
                !stat = matmult_lop(adj=.FALSE.,add=.FALSE.,x=x,y=linRegForward)
                do i=1, size(x)
                        linRegForward(i) = aa(i,1)*modl(1)+aa(i,2)*modl(2)
                end do
        end function
end program
