/*
Author: Stuart Farris
Date: 25Aug2016
Description: pick desired bandwidth for slope picking by fixing badwidth to depth. Lower bound set to index 30.

Use: ./bandPick1 < inFile > outFile
inFile: n1=mx n2=win n3=freq
outFile: n1=mx n2=win n3=bandLimits
*/

#include <seplib.h>
#include <string>

int main(int argc, char* argv[])
{
	/*initialize IO*/
	initpar(argc,argv);
	
	/* declare variables*/
	int 	n1,n2,n3;
	float 	o1,o2,o3;
	float 	d1,d2,d3;
	float 	*data;
	float 	uuBoundFreq, ulBoundFreq; //upper and lower limits of the max freq
	int 	uuBoundIndex, ulBoundIndex; //index of the upper and lower bounds of max freq
	
	int newn3=2;
	int newo3=0;
	int newd3=1;
	std::string newlabel3="bounds";

	/*retrieve header info and read in data*/	
	//Get values from header file
	if(!hetch("n1","d",&n1)) seperr("could not get n1 from file\n");
	if(!hetch("n2","d",&n2)) seperr("could not get n2 from file\n");
	if(!hetch("n3","d",&n3)) seperr("could not get n3 from file\n");
	if(!hetch("o3","f",&o3)) seperr("could not get o3 from file\n");
	if(!hetch("d3","f",&d3)) seperr("could not get d3 from file\n");
	//read in data
	data = new float [n1*n2*n3];
	if(!sreed("in",data,sizeof*data*n1*n2*n3)) seperr("could not read data\n");
	else fprintf(stderr,"sreed successful\n");

	/*Look at each window and calculate bounds of usable freq*/
	uuBoundFreq=.04;
	ulBoundFreq=.02;
	uuBoundIndex=uuBoundFreq/d3;
	ulBoundIndex=ulBoundFreq/d3;
	float *n3Bounds = new float[n1*n2*newn3];	

	for( int in1=0; in1<n1; in1++){
		for( int in2=0; in2<n2; in2++){

			n3Bounds[0*n2*n1+in2*n1+in1]=30;
			n3Bounds[1*n2*n1+in2*n1+in1]=(n2-in2)*(uuBoundIndex-ulBoundIndex)/n2+ulBoundIndex;
			//fprintf(stderr,"n1=%d\tn2=%d\tmin: %f \tmax: %f\n",in1,in2,n3Bounds[0*n2*n1+in2*n1+in1],n3Bounds[1*n2*n1+in2*n1+in1]);
		}
	}
	
	/*write out freq bounds to new file*/

	if(putch("n3","d",&newn3)) seperr("Could not write n1 to file");
	if(putch("o3","d",&newo3))   seperr("Could not write o3 to file");
	if(putch("d3","d",&newd3))   seperr("Could not write d3 to file");
	if(putch("label3","s",&newlabel3))   seperr("Could not write label3 to file");

	if(!srite("out",n3Bounds,sizeof*n3Bounds*n1*n2*newn3)) seperr("Could not write out data");	
	
	//cleanup
	delete[] data;
	data = NULL;
	
	delete[] n3Bounds;
	n3Bounds = NULL;
	

	return 0;
}
