module prob_comb_sum_mod
! Adding two problem using one module
! Last edited by Ali Almomin 08/24/2015

  use prob_obj_mod

  implicit none

  private
  public :: prob_comb_sum
!-------------------- problem object --------------------
  type, extends(prob_obj)             :: prob_comb_sum
    private
    class(prob_obj), pointer          :: prob1   ! first problem pointer
    class(prob_obj), pointer          :: prob2   ! second problem pointer
    real, public                      :: eps     ! scalar for second problem
    real, dimension(:), allocatable   :: tempm   ! temporary model array
  contains
    private
    procedure                         :: objf    ! objective function
    procedure                         :: resf    ! residual function
    procedure                         :: gradf   ! gradient function
    procedure                         :: dresf   ! residual change function
  end type
!-------------------- interface block --------------------
  interface prob_comb_sum
    module procedure const         ! User-defined constructor
  end interface

  contains
!-------------------- constructor function --------------------
  type(prob_comb_sum) function const(prob1, prob2, eps, mmov)
    class(prob_obj), target           :: prob1
    class(prob_obj), target           :: prob2
    real                              :: eps
    character(256)                    :: mmov
    integer                           :: stat
    const%prob1 => prob1
    const%prob2 => prob2
    const%eps = eps
    if(prob1%nmodl .ne. prob2%nmodl) call erexit("ERROR: prob1%nmodl is different than prob2:nmodl")
    const%nmodl = prob1%nmodl
    const%ndata = prob1%ndata + prob2%ndata
    allocate(const%tempm(prob1%nmodl))
    const%mmov = mmov
    stat = const%alloc()
  end function
!-------------------- objective function --------------------
  integer function objf(this, res, obj)
    class(prob_comb_sum)              :: this
    real, dimension(:)                :: res             ! residual (input)
    double precision                  :: obj             ! objective function value (output)
    double precision                  :: obj1             ! objective function value (output)
    double precision                  :: obj2             ! objective function value (output)
    integer                           :: stat            ! status flag
    stat = this%prob1%objf(res(1:this%prob1%ndata), obj1)
    stat = this%prob2%objf(res(this%prob1%ndata+1:this%ndata), obj2)
    obj = obj1 + obj2
    objf = 0
  end function
!-------------------- residual function --------------------
  integer function resf(this, modl, res)
    class(prob_comb_sum)              :: this
    real, dimension(:)                :: modl            ! model (input)
    real, dimension(:)                :: res             ! residual (output)
    integer                           :: stat            ! status flag
    stat = this%prob1%resf(modl, res(1:this%prob1%ndata))
    stat = this%prob2%resf(modl, res(this%prob1%ndata+1:this%ndata))
    res(this%prob1%ndata+1:this%ndata) = this%eps*res(this%prob1%ndata+1:this%ndata)
    resf = 0
  end function
!-------------------- gradient function --------------------
  integer function gradf(this, modl, res, grad)
    class(prob_comb_sum)              :: this
    real, dimension(:)                :: modl            ! model (input)
    real, dimension(:)                :: res             ! residual (input)
    real, dimension(:)                :: grad            ! gradient (output)
    integer                           :: stat   ! status flag
    stat = this%prob1%gradf(modl, res(1:this%prob1%ndata), grad)
    stat = this%prob2%gradf(modl, res(this%prob1%ndata+1:this%ndata), this%tempm)
    grad = grad + this%eps*this%tempm
    gradf = 0
  end function
!-------------------- residual change function --------------------
  integer function dresf(this, modl, dmodl, dres)
    class(prob_comb_sum)              :: this
    real, dimension(:)                :: modl            ! model (input)
    real, dimension(:)                :: dmodl            ! gradient (input)
    real, dimension(:)                :: dres           ! gradient in data space (output)
    integer                           :: stat            ! status flag
    stat = this%prob1%dresf(modl, dmodl, dres(1:this%prob1%ndata))
    stat = this%prob2%dresf(modl, dmodl, dres(this%prob1%ndata+1:this%ndata))
    dres(this%prob1%ndata+1:this%ndata) = this%eps*dres(this%prob1%ndata+1:this%ndata)
    dresf = 0
  end function

end module
