module solver_lbfgs_mod
! LBFGS solver
! Last edited by Ali Almomin 08/24/2015

  use solver_obj_mod
  use prob_obj_mod
  use stepper_obj_mod
  use terminate_obj_mod
  use lbfgs_mod

  implicit none

  private
  public :: solver_lbfgs

  type, extends(solver_obj)         :: solver_lbfgs
    integer                         :: m = 20
    integer                         :: iprint = 0
    character(len=60)               :: csave = ""
    logical                         :: lsave(4) = .false.
    integer                         :: isave(44) = 0
    double precision                :: dsave(29) = 0.0d0
    double precision                :: factr  = 0.0d0
    double precision                :: pgtol  = 0.0d0
    contains
    procedure                       :: run
  end type

  contains

  integer function run(this, modl, prob, stpr, term)
    class(solver_lbfgs)             :: this
    class(prob_obj)                   :: prob
    class(stepper_obj)              :: stpr
    class(terminate_obj)                :: term
    !-------------------- interface variables -------
    real, dimension(:)              :: modl
    integer                         :: n, m
    integer                         :: iter
    integer                         :: stat
    character(len=60)               :: task
    double precision                :: f
    integer, allocatable            :: nbd(:), iwa(:)
    real, dimension(:), allocatable :: grad
    double precision, allocatable   :: x(:), l(:), u(:), g(:), wa(:)

    n = prob%nmodl
    m = this%m

    allocate(grad(n))
    allocate(nbd(n))
    allocate(x(n))
    allocate(l(n))
    allocate(u(n))
    allocate(g(n))
    allocate(iwa(3*n))
    allocate(wa(2*m*n+5*n+11*m*m+8*m))

    nbd = 0
    if(associated(stpr%mmin)) then
      nbd = 1
      l = stpr%mmin
    end if
    if(associated(stpr%mmax)) then
      nbd = 3 - nbd
      u = stpr%mmax
    end if

    iter = 1
    task = 'START'

    do while(task(1:2) .eq. 'FG' .or. task .eq. 'NEW_X' .or. task .eq. 'START')
      if(task .eq. 'START' .or. task(1:5) .eq. 'NEW_X') stat = prob%output(modl)
      call setulb(n,m,x,l,u,nbd,f,g,this%factr,this%pgtol,wa,iwa,task,this%iprint,this%csave,this%lsave,this%isave,this%dsave)
      if(task(1:2) .eq. 'FG') then
        modl = real(x)
        stat = prob%get_obj(modl, f)
        stat = prob%get_grad(modl, grad)
        g = grad
      else if(task(1:5) .eq. 'NEW_X') then
        stat = prob%get_obj(modl, f)
        write(0,*) "iter", iter, " obj =", real(f), " feval =", prob%get_fevals()
        iter = iter + 1
        if(term%test(iter, prob)) exit
      else if(task(1:4) .eq. 'STOP') then
        write (0,*) "Stopping: ", task
      end if
    end do

    deallocate(grad)
    deallocate(nbd)
    deallocate(x)
    deallocate(l)
    deallocate(u)
    deallocate(g)
    deallocate(iwa)
    deallocate(wa)

    run = 0
  end function

end module
