module solver_lcg_mod
! Jon's inear CG solver
! Last edited by Ali Almomin 08/24/2015

  use solver_obj_mod
  use prob_obj_mod
  use stepper_obj_mod
  use terminate_obj_mod

  implicit none

  private
  public :: solver_lcg

  type, extends(solver_obj)         :: solver_lcg !extending solver to lcd
    contains
    procedure                       :: run     !Function to run the solver
  end type

  contains

  integer function run(this, modl, prob, stpr, term)
    class(solver_lcg)               :: this
    !-------------------- interface variables -------
    integer                         :: stat
    real, dimension(:)              :: modl            ! model (input/output)
    class(prob_obj)                 :: prob
    class(stepper_obj)              :: stpr
    class(terminate_obj)                :: term
    double precision                :: alpha, beta, f1, f0
    double precision                :: sds, gdg, gds, determ, gdr, sdr
    integer                         :: iter
    logical                         :: success
    real, dimension(:), allocatable :: grad, gradd, res, dmodl, dres                 !

    allocate(grad(prob%nmodl))
    allocate(dmodl(prob%nmodl))
    allocate(gradd(prob%ndata))
    allocate(res(prob%ndata))
    allocate(dres(prob%ndata))

    success = .true.
    iter = 1
    do
      stat = prob%get_obj(modl, f0)
      stat = prob%get_res(modl, res)
      stat = prob%get_grad(modl, grad)
      stat = prob%get_dres(modl, grad, gradd)
      stat = prob%output(modl)
      if(iter .eq. 1) then
        dmodl = 0.
        dres = 0.
        beta = 0.d0    ! steepest descent
        if(sum(dprod(gradd, gradd)) .eq. 0) success = .false.
        alpha = - sum(dprod(gradd, res)) / sum(dprod(gradd, gradd))
      else
        gdg = sum(dprod(gradd, gradd))
        sds = sum(dprod(dres, dres))
        gds = sum(dprod(gradd, dres))
        if(gdg .eq. 0. .or. sds .eq. 0.) success = .false.
        determ = gdg * sds * max(1.d0 - (gds/gdg)*(gds/sds), 1.d-12)
        gdr = - sum(dprod(gradd, res))
        sdr = - sum(dprod(dres, res))
        alpha = (sds * gdr - gds * sdr) / determ
        beta = (-gds * gdr + gdg * sdr) / determ
      end if
      if(.not. success) then
        write(0,*) "Couldn't find a proper step size, will terminate solver"
        exit
      end if
      dmodl = alpha * grad + beta * dmodl            ! update solution step
      dres  = alpha * gradd + beta * dres            ! update solution step
      modl = modl + dmodl                           ! update solution
      stat = stpr%clip_modl(modl)
      stat = prob%get_obj(modl, f1)
      if(f1 >= f0) then
        write(0,*) "Objective function didn't reduce, will terminate solver"
        modl = modl - dmodl
        exit
      end if
      write(0,*) "iter", iter, " obj =", real(f1), " feval =", prob%get_fevals()
      iter = iter + 1
      if(term%test(iter, prob)) exit
    end do

    deallocate(grad)
    deallocate(gradd)
    deallocate(res)
    deallocate(dmodl)
    deallocate(dres)

    run = 0
  end function

end module
