module stepper_backtrack_mod
! Backtracking stepper
! Last edited by Ali Almomin 08/24/2015

  use stepper_obj_mod
  use prob_obj_mod

  implicit none

  private
  public :: stepper_backtrack

  type, extends(stepper_obj) :: stepper_backtrack ! extend line search to back tracking method
  !Variable extension
    double precision                  :: decrease_coef = 0.01       !
    double precision                  :: curvatur_coef = 0.5       !
    double precision                  :: alpha = 0.
    double precision                  :: alpha_min = 0.25
    double precision                  :: alpha_max = 4
    double precision                  :: rho = 0.5
    real, dimension(:), allocatable   :: modl, res, dres                 ! internal copy of m+alpha*p
    real, dimension(:), allocatable   :: grad                 ! internal copy of gradient
  contains
    procedure                         :: run
  end type

  contains

  !Calling function to choose and run a solver
  integer function run(this, modl, prob, dmodl, alpha, success)
    class(stepper_backtrack)          :: this
    class(prob_obj)                     :: prob
    real,dimension(:)                 :: dmodl           ! search direction (input)
    real,dimension(:)                 :: modl            ! model (input/output)
    double precision                  :: alpha           ! step size (output)
    double precision                  :: alpha_min
    double precision                  :: alpha_max
    logical                           :: success
    integer                           :: stat
    double precision                  :: phi_0               !
    double precision                  :: phi_p               !
    double precision                  :: f0
    double precision                  :: fp

    if(.not. allocated(this%modl)) allocate(this%modl(prob%nmodl))
    if(.not. allocated(this%grad)) allocate(this%grad(prob%nmodl))
    if(.not. allocated(this%res)) allocate(this%res(prob%ndata))
    if(.not. allocated(this%dres)) allocate(this%dres(prob%ndata))
    success = .false.
    alpha = this%alpha
    if(alpha .eq. 0.) then
      stat = prob%get_dres(modl, dmodl, this%dres)
      stat = prob%get_res(modl, this%res)
      alpha = - sum(dprod(this%dres, this%res)) / sum(dprod(this%dres, this%dres))
    end if
    alpha_min = this%alpha_min*alpha
    alpha_max = this%alpha_max*alpha

    ! run the stepper
    stat = prob%get_obj(modl, f0)
    stat = prob%get_grad(modl, this%grad)
    phi_0 = sum(dprod(this%grad,dmodl))      ! phi(0)
    phi_p = phi_0                      ! make sure we enter the while loop
    alpha = alpha_max
    this%modl = modl + alpha*dmodl
    stat = this%clip_modl(this%modl)
    stat = prob%get_obj(this%modl, fp)
    do while( fp > f0 + this%decrease_coef*alpha*phi_0 .or. phi_p < this%curvatur_coef*phi_0)
      alpha = alpha*this%rho   ! backtrack
      this%modl  = modl + alpha*dmodl
      stat = this%clip_modl(this%modl)
      stat = prob%get_obj(this%modl, fp)
      stat = prob%get_grad(this%modl, this%grad)
      phi_p= sum(dprod(this%grad,dmodl))
      if(alpha < alpha_min) then
        exit
      end if
    end do
    if(fp <= f0 + this%decrease_coef*alpha*phi_0 .and. phi_p >= this%curvatur_coef*phi_0) then
      success = .true.
      this%alpha = alpha
      modl = this%modl
    end if

    run = 0
  end function

end module
