module stepper_linear_mod
! Linear stepper
! Last edited by Ali Almomin 08/24/2015

  use stepper_obj_mod
  use prob_obj_mod

  implicit none

  private
  public :: stepper_linear

  type, extends(stepper_obj)          :: stepper_linear ! extend line search to back tracking method
  !Variable extension
    real, dimension(:), allocatable   :: modl, res, dres                 !
  contains
    procedure                         :: run
  end type

  contains

  !Calling function to choose and run a solver
  integer function run(this, modl, prob, dmodl, alpha, success)
    class(stepper_linear)             :: this
    class(prob_obj)                   :: prob
    real,dimension(:)                 :: dmodl           ! search direction (input)
    real,dimension(:)                 :: modl            ! model (input/output)
    double precision                  :: alpha           ! step size (output)
    integer                           :: stat
    logical                           :: success
    double precision                  :: f0
    double precision                  :: f1

    if(.not. allocated(this%modl)) allocate(this%modl(prob%nmodl))
    if(.not. allocated(this%res)) allocate(this%res(prob%ndata))
    if(.not. allocated(this%dres)) allocate(this%dres(prob%ndata))
    success = .false.

    stat = prob%get_obj(modl, f0)
    stat = prob%get_res(modl, this%res)
    stat = prob%get_dres(modl, dmodl, this%dres)
    alpha = - sum(dprod(this%dres, this%res)) / sum(dprod(this%dres, this%dres))
    this%modl = modl + alpha*dmodl
    stat = this%clip_modl(this%modl)
    stat = prob%get_obj(this%modl, f1)
    if(f1 < f0) then
      success = .true.
      modl = this%modl
    end if
    run = 0
  end function

end module
