module stepper_obj_mod
! Stepper abstract object
! Last edited by Ali Almomin 08/24/2015

  use prob_obj_mod

  implicit none

  private
  public :: stepper_obj

  type, abstract  :: stepper_obj
  real, dimension(:), pointer                         :: mmin => null()     ! model lower bound
  real, dimension(:), pointer                         :: mmax => null()     ! model upper bound
  contains
    procedure(stepper_function), deferred :: run
    procedure                             :: clip_modl
  end type

  abstract interface
    integer function stepper_function(this, modl, prob, dmodl, alpha, success)
      import                              :: stepper_obj
      import                              :: prob_obj
      !-------------------- interface variables -------
      class(stepper_obj)                  :: this
      class(prob_obj)                       :: prob
      real,dimension(:)                   :: dmodl           ! search direction (input)
      real,dimension(:)                   :: modl            ! model (input/output)
      double precision                    :: alpha           ! step size (output)
      logical                             :: success  !
    end function
  end interface

  contains

  integer function clip_modl(this, modl)
    class(stepper_obj)          :: this
    real, dimension(:)          :: modl
    if(associated(this%mmin)) where(modl < this%mmin) modl = this%mmin
    if(associated(this%mmax)) where(modl > this%mmax) modl = this%mmax
    clip_modl = 0
  end function

end module
