module stepper_parab_mod
! Parabola fitting stepper
! Last edited by Ali Almomin 08/24/2015

  use stepper_obj_mod
  use prob_obj_mod

  implicit none

  private
  public :: stepper_parab

  type, extends(stepper_obj)          :: stepper_parab ! extend line search to back tracking method
  !Variable extension
    integer                           :: c1 = 1.0
    integer                           :: c2 = 2.0
    double precision                  :: ntry = 10
    double precision                  :: alpha = 0.
    double precision                  :: alpha_min = 0.25
    double precision                  :: alpha_max = 4.0
    double precision                  :: r = 0.25
    real, dimension(:), allocatable   :: modl, res, dres                 !
  contains
    procedure                         :: run
  end type

  contains

  !Calling function to choose and run a solver
  integer function run(this, modl, prob, dmodl, alpha, success)
    class(stepper_parab)              :: this
    class(prob_obj)                   :: prob
    real,dimension(:)                 :: dmodl           ! search direction (input)
    real,dimension(:)                 :: modl            ! model (input/output)
    double precision                  :: alpha, a, b, xv           ! step size (output)
    integer                           :: stat
    integer                           :: i
    logical                           :: success
    double precision                  :: f0
    double precision                  :: f1
    double precision                  :: f2
    double precision                  :: f3

    if(.not. allocated(this%modl)) allocate(this%modl(prob%nmodl))
    if(.not. allocated(this%res)) allocate(this%res(prob%ndata))
    if(.not. allocated(this%dres)) allocate(this%dres(prob%ndata))
    success = .false.
    stat = prob%get_obj(modl, f0)
    alpha = this%alpha

    ! run the stepper
    do i=1,2*this%ntry
      if(alpha .eq. 0. .or. i .eq. this%ntry) then
        stat = prob%get_dres(modl, dmodl, this%dres)
        stat = prob%get_res(modl, this%res)
        alpha = - sum(dprod(this%dres, this%res)) / sum(dprod(this%dres, this%dres))
      end if
      this%modl = modl + this%c1*alpha*dmodl
      stat = this%clip_modl(this%modl)
      stat = prob%get_obj(this%modl, f1)
      this%modl = modl + this%c2*alpha*dmodl
      stat = this%clip_modl(this%modl)
      stat = prob%get_obj(this%modl, f2)

      if(f0 .eq. f1 .and. f0 .eq. f2) then
        xv = this%alpha_min
      else
        xv = (this%c2*this%c2*(f1-f0) + this%c1*this%c1*(f0-f2)) / (2*this%c2*(f1-f0) + 2*this%c1*(f0-f2))
      end if
      xv = max(xv,this%alpha_min)
      xv = min(xv,this%alpha_max)
      this%modl = modl + xv*alpha*dmodl
      stat = this%clip_modl(this%modl)
      stat = prob%get_obj(this%modl, f3)

      if(f1 < f0 .and. f1 < f2 .and. f1 < f3) then
        alpha = alpha * this%c1
        success = .true.
        exit
      else if(f2 < f0 .and. f2 < f1 .and. f2 < f3) then
        alpha = alpha * this%c2
        success = .true.
        exit
      else if(f3 < f0 .and. f3 < f1 .and. f3 < f2) then
        alpha = alpha * xv
        success = .true.
        exit
      else
        alpha = alpha * this%r
      end if
    end do
    if(success) then
      ! finish line search, update model
      this%alpha = alpha
      modl = modl + alpha*dmodl
      stat = this%clip_modl(modl)
    end if
    run = 0
  end function

end module
