module stepper_sample_mod
! Sampling stepper
! Last edited by Ali Almomin 08/24/2015

  use stepper_obj_mod
  use prob_obj_mod

  implicit none

  private
  public :: stepper_sample

  type, extends(stepper_obj)          :: stepper_sample ! extend line search to back tracking method
  !Variable extension
    integer                                     :: n = 3
    real                                        :: cmin = 0.5
    real                                        :: cmax = 1.5
    double precision                            :: ntry = 10
    double precision                            :: alpha = 0.
    double precision                            :: r = 0.25
    real, dimension(:), allocatable             :: modl
    real, dimension(:), allocatable             :: res
    real, dimension(:), allocatable             :: dres
    real, dimension(:), allocatable             :: c
    double precision, dimension(:), allocatable :: f
  contains
    procedure                                   :: run
  end type

  interface stepper_sample
    module procedure const
  end interface

  contains

  type(stepper_sample) function const(n, cmin, cmax, ntry, alpha, r)
    optional                       :: n, cmin, cmax, ntry, alpha, r
    integer                     :: n
    real                        :: cmin
    real                        :: cmax
    double precision            :: ntry
    double precision            :: alpha
    double precision            :: r
    integer                     :: i
    if(present(n)) const%n = n
    if(present(cmin)) const%cmin = cmin
    if(present(cmax)) const%cmax = cmax
    if(present(ntry)) const%ntry = ntry
    if(present(alpha)) const%alpha = alpha
    if(present(r)) const%r = r
    allocate(const%c(const%n))
    allocate(const%f(const%n))
    do i=1,const%n
      const%c(i) = (i-1.)*(const%cmax-const%cmin)/(const%n-1.) + const%cmin
    end do
  end function

  !Calling function to choose and run a solver
  integer function run(this, modl, prob, dmodl, alpha, success)
    class(stepper_sample)             :: this
    real,dimension(:)                 :: modl            ! model (input/output)
    class(prob_obj)                   :: prob
    real,dimension(:)                 :: dmodl           ! search direction (input)
    double precision                  :: alpha           ! step size (output)
    logical                           :: success
    integer                           :: stat
    double precision                  :: f0
    integer                           :: i
    integer                           :: itry

    if(.not. allocated(this%modl)) allocate(this%modl(prob%nmodl))
    if(.not. allocated(this%res)) allocate(this%res(prob%ndata))
    if(.not. allocated(this%dres)) allocate(this%dres(prob%ndata))
    success = .false.
    stat = prob%get_obj(modl, f0)
    alpha = this%alpha

    ! run the stepper
    do itry=1,2*this%ntry
      if(alpha .eq. 0. .or. itry .eq. this%ntry) then
        stat = prob%get_dres(modl, dmodl, this%dres)
        stat = prob%get_res(modl, this%res)
        alpha = - sum(dprod(this%dres, this%res)) / sum(dprod(this%dres, this%dres))
      end if
      do i=1,this%n
        this%modl = modl + this%c(i)*alpha*dmodl
        stat = this%clip_modl(this%modl)
        stat = prob%get_obj(this%modl, this%f(i))
      end do
      if(minval(this%f) < f0) then
        alpha = alpha * this%c(sum(minloc(this%f)))
        success = .true.
        exit
      else
        alpha = alpha * this%r
      end if
    end do
    if(success) then
      ! finish line search, update model
      this%alpha = alpha
      modl = modl + alpha*dmodl
      stat = this%clip_modl(modl)
    end if
    run = 0
  end function

end module
