module terminate_simple_mod
! Simple terminator
! Last edited by Ali Almomin 08/24/2015

  use terminate_obj_mod
  use prob_obj_mod

  implicit none

  private
  public :: terminate_simple

  type, extends(terminate_obj)  :: terminate_simple
    integer                     :: niter = 0              ! maximum iterations allowed (input)
    integer                     :: maxfevals = 0      ! maximum function evaluations allowed (input)
    real                        :: maxhours = 0.       ! maximum hours allowed (input)
    double precision            :: tolr = 1.0d-20      ! residual norm tolerance (input)
    double precision            :: tolg = 1.0d-20      ! gradient norm tolerance (input)
    integer(kind=8), private    :: timer0 = 0
    integer(kind=8), private    :: rate = 0
  contains
    procedure                   :: test
  end type

  interface terminate_simple
    module procedure const
  end interface

  contains

  type(terminate_simple) function const(niter, maxfevals, maxhours, tolr, tolg)
    optional                     :: niter, maxfevals, maxhours, tolr, tolg
    integer                     :: niter
    integer                     :: maxfevals
    real                        :: maxhours
    double precision            :: tolr
    double precision            :: tolg
    if(present(niter)) const%niter = niter
    if(present(maxfevals)) const%maxfevals = maxfevals
    if(present(maxhours)) const%maxhours = maxhours
    if(present(tolr)) const%tolr = tolr
    if(present(tolg)) const%tolg = tolg
    call system_clock(const%timer0, const%rate)
    const%rate = const%rate * 3600.
  end function

  logical function test(this, iter, prob)
    class(terminate_simple)           :: this
    integer                     :: iter
    class(prob_obj)             :: prob
    real                        :: hours_passed
    integer(kind=8)             :: timer1
    test = .false.
    if(this%niter > 0 .and. iter > this%niter) then
      test = .true.
      write(0,*) "Terminate: maximum number of iterations reached"
    end if
    if(this%maxfevals > 0 .and. prob%get_fevals() >= this%maxfevals) then
      test = .true.
      write(0,*) "Terminate: maximum number of evaluations"
    end if
    if(this%maxhours > 0.) then
      call system_clock(timer1)
      hours_passed = 1.*(timer1-this%timer0)/this%rate
      if(hours_passed >= this%maxhours) then
        test = .true.
        write(0,*) "Terminate: maximum number hours reached",hours_passed, this%maxhours
      end if
    end if
    if(prob%get_rnorm() < this%tolr) then
      test = .true.
      write(0,*) "Terminate: minimum residual tolerance reached", real(prob%get_rnorm()), real(this%tolr)
    end if
    if(prob%get_gnorm() < this%tolg) then
      test = .true.
      write(0,*) "Terminate: minimum gradient tolerance reached", real(prob%get_gnorm()), real(this%tolg)
    end if
  end function

end module
